<?php
namespace E_connector_For_Woocommerce;
/**
 * Class for handling frontend account page.
 */
class Account_Page implements Hookable {

    /**
     * Account page endpoints to register
     */
    protected $endpoints = array();

    /**
     * Constructor
     */
    public function __construct(){
        $this->endpoints = array(
            'documents' => array(
                'label'    => __( 'Documents', 'e-connector-for-woocommerce' ),
                'after'    => 'downloads',
                'callback' => array( $this, 'documents_callback' ),
                'endpoint' => 'documents',
            ),
        );
    }

    /**
     * Registers hooks
     */
    public function register_hooks(){
        add_filter( 'woocommerce_account_menu_items', array( $this, 'account_links' ), 10, 1 );
        add_action( 'init', array( $this, 'register_account_endpoints' ), 10, 1 );
        add_action( 'e_connector_for_woocommerce_documents_table_column_data', array( $this, 'documents_table_column_data' ), 10, 2 );
    }

    /**
     * Returns the account endpoints to register.
     */
    function get_account_endpoints(){
        return apply_filters( 'e_connector_for_woocommerce_account_endpoints', $this->endpoints );
    }

    /**
     * Filters the account links
     */
    public function account_links( $links ){
        $endpoints = $this->get_account_endpoints();
        foreach ( $endpoints as $slug => $data ) {
            $links = e_connector_for_woocommerce_array_insert( $links, $data['after'], array( $slug => $data['label'] ) );
        }
        return $links;
    }

    /**
     * Registers new endpoints for the account tabs.
     */
    function register_account_endpoints() {
        $endpoints = $this->get_account_endpoints();
        foreach ( $endpoints as $slug => $data ) {
            add_rewrite_endpoint( $slug, EP_PAGES );
            add_action( "woocommerce_account_${slug}_endpoint", $data['callback'] );
        }
    }

    /**
     * Account page content for the 'documents/' endpoint
     */
    function documents_callback() {
        $user_id   = get_current_user_id();
        $documents = get_user_meta( $user_id, '_econnector_documents', true );
        if( is_array( $documents ) ) $documents = array_reverse( $documents );
        $columns   = apply_filters( 'e_connector_for_woocommerce_document_table_headers', array(
            'document-name'   => __( 'Document name', 'e-connector-for-woocommerce' ),
            'type'            => __( 'Type', 'e-connector-for-woocommerce' ),
            'reference'       => __( 'Reference', 'e-connector-for-woocommerce' ),
            'total_paid'      => __( 'Total paid', 'e-connector-for-woocommerce' ),
            'date_created'    => __( 'Date', 'e-connector-for-woocommerce' ),
            'document-actions' => __( 'Actions', 'e-connector-for-woocommerce' ),
        ) );
        wc_get_template( 
            'myaccount/documents.php', 
            array(
                'user_id'    => (int) $user_id,
                'documents'  => $documents,
                'columns'    => $columns,
            ),
            '',
            E_CONNECTOR_FOR_WOOCOMMERCE_TEMPLATES_PATH
        );
    }

    /**
     * Documents table column content
     * 
     * @param   int     $document_id  ID of the current document
     * @param   string  $column_key   Column being displayed
     * @return  string  $html         HTML to display
     */
    function documents_table_column_data( $document_id, $column_key ){
        $html = '';
        switch ( $column_key ) {
            case 'document-name':
                $html = esc_html( basename( get_attached_file( $document_id ) ) );
                break;
            case 'type':
                $type  = get_post_meta( $document_id, '_document_type', true );
                $label = e_connector_for_woocommerce_get_document_type_label( $type );
                $html  = esc_html( $label );
                break;
            case 'reference':
                $reference = get_post_meta( $document_id, '_reference', true );
                $html = ! empty( $reference ) ? esc_html( $reference ) : '';
                break;
            case 'total_paid':
                $total_paid = get_post_meta( $document_id, '_total_paid', true );
                $html = ! empty( $total_paid ) ? esc_html( $total_paid ) : '';
                break;
            case 'date_created':
                $timestamp = get_the_date( 'U', $document_id );
                $date_W3C  = wp_date( DATE_W3C, $timestamp );
                $date      = wp_date( get_option( 'date_format' ), $timestamp );
                $time      = wp_date( get_option( 'time_format' ), $timestamp );
                $html      = sprintf(
                    /* translators: %1$s: document date, %2$s: document date, %3$s: document time */ 
                    _x( 'On <time datetime="%1$s">%2$s at %3$s</time>', 'Documents table date column', 'e-connector-for-woocommerce' ),
                    $date_W3C,
                    $date,
                    $time
                );
                break;
            case 'document-actions':
                $actions = $this->get_document_actions( $document_id );
                $html    = '';
                if( $actions ){
                    $links = '';
                    foreach ( $actions as $key => $action ) {
                        $icon  = ! empty( $action['icon'] ) ? $action['icon'] : '';
                        $label = sprintf( '%s<span class="document-action-label">%s</span>', $icon, esc_html( $action['label'] ) );
                        $links .= sprintf( '<li class="document-action"><a href="%s" target="_blank" rel="noopener noreferer">%s</a></li>', esc_url( $action['url'] ), $label ); 
                    }
                    if( $links ){
                        $html = sprintf('<ul class="document-actions">%s</ul>', $links );
                    }
                }
                break;
        }
        $html = apply_filters( 'e_connector_for_woocommerce_document_table_column_data_html', $html, $document_id, $column_key );
        echo wp_kses_post( $html );
        return $html;
    }

    /**
     * Returns an array of document actions
     * 
     * @param   int    $document_id 
     * @return  array  $actions  Document actions.
     */
    public function get_document_actions( $document_id ){
        $admin_url = admin_url( 'admin-post.php' );
        $download_url = add_query_arg( array(
            'action'      => 'download_document',
            'document_id' => (int) $document_id,
            '_wp_nonce'   => wp_create_nonce( 'download_document' ),
        ), $admin_url );
        $view_url = add_query_arg( array(
            'action'      => 'view_document',
            'document_id' => (int) $document_id,
            '_wp_nonce'   => wp_create_nonce( 'view_document' ),
        ), $admin_url );
        $actions = array(
            'download-document' => array(
                'action' => 'download-document',
                'url'    => $download_url,
                'label'  =>  __( 'Download', 'e-connector-for-woocommerce' ) ,
                'icon'   => e_connector_for_woocommerce_get_icon( 'download' ),
            ),
            'view-document' => array(
                'action' => 'view-document',
                'url'    => $view_url,
                'label'  =>  __( 'View', 'e-connector-for-woocommerce' ) ,
                'icon'   => e_connector_for_woocommerce_get_icon( 'view' ),
            ),
        );
        return apply_filters( 'e_connector_for_woocommerce_document_actions', $actions, $document_id );
    }
}