<?php
namespace E_connector_For_Woocommerce;
/**
 * Class for handling all content-related actions.
 */
class Content_Manager implements Hookable {

    /**
     * Registers hooks
     */
    public function register_hooks(){
        add_filter( 'query_vars', array( $this, 'query_vars' ), 10, 1 );
        add_action( 'pre_get_posts', array( $this, 'pre_get_posts' ), 10, 1 );
        add_action( 'template_redirect', array( $this, 'template_redirect' ), 10 );
        add_action( 'delete_attachment', array( $this, 'handle_document_delete' ), 10, 2 );
        add_action( 'restrict_manage_posts', array( $this, 'document_type_attachment_filter' ), 10, 2 );
        add_action( 'admin_post_download_document', array( $this, 'handle_document_action' ), 10, 2 );
        add_action( 'admin_post_view_document', array( $this, 'handle_document_action' ), 10, 2 );
    }

    /**
     * Registers new query vars
     * 
     * @param   array  $query_vars  Registered query vars
     * @return  array  $query_vars
     */
    public function query_vars( $query_vars ) {
        if( is_admin() ){
            $query_vars[] = 'document-type';
        }
        return $query_vars;
    }

    /**
     * Filters the query on media admin page
     * 
     * @param  WP_Query  $query
     */
    public function pre_get_posts( $query ){
        if( is_admin() && 'attachment' === $query->get( 'post_type' ) ){
            $document_type = $query->get( 'document-type' );
            if( ! $document_type ) return;
    
            $meta_query = $query->get( 'meta_query' );
            if( empty( $meta_query ) ){
                $meta_query = array();
            }
            $meta_query[] = array(
                'key'     => '_document_type',
                'value'   => (int) $document_type
            );
            $query->set('meta_query', $meta_query );
        }
    }

    /**
     * Redirects user to home page if trying to view another user's documents
     */
    function template_redirect(){
        if( is_singular( 'attachment' ) ){
            $post_id = (int) get_the_ID();
            $owner   = get_post_meta( $post_id, '_econnector_owner', true );
            // if( $owner && ! e_connector_for_woocommerce_is_owner() && ! current_user_can( 'manage_options' ) ){
            if( $owner && ! current_user_can( 'manage_options' ) ){
                wp_safe_redirect( home_url() );
                exit;
            }
        }
    }

    /**
     * Removes documents from owner's documents when it is deleted.
     * 
     * @param  int      $post_id  Attachment id
     * @param  WP_Post  $post     Attachment
     */
    public function handle_document_delete( $post_id, $post = false ){
        if( ! $post ){
            $post = get_post( $post_id );
        }
        if ( 'attachment' !== $post->post_type ) return false;
        
        $owner_id = get_post_meta( $post_id, '_econnector_owner', true );
        if( ! $owner_id ) return false;
        
        $owner_files = get_user_meta( $owner_id, '_econnector_documents', true );
        if( empty( $owner_files ) ) return false;
        
        if( in_array( (int) $post_id, $owner_files ) ){
            $owner_files = array_values( array_filter( $owner_files, function( $file_id ) use ( $post_id ) {
                return (int) $file_id !== (int) $post_id;
            } ) );
			update_user_meta( $owner_id, '_econnector_documents', $owner_files );
        }
        return true;
    }

    /**
     * Adds a document type filter on Media admin page
     * 
     * @param  string  $post_type  Current post type
     * @param  string  $which      Which bar to add filter to.
     */
    public function document_type_attachment_filter( $post_type, $which){
        if( 'attachment' !== $post_type || 'bar' !== $which ){
            return;
        }
        $selected = ! empty( $_GET['document-type'] ) ? (int) $_GET['document-type'] : '';
        $document_types = e_connector_for_woocommerce_get_document_types();
        ?>
            <label for="filter-by-document-type" class="screen-reader-text"><?php esc_html_e( 'Filter by document type', 'e-connector-for-woocommerce' ); ?></label>
            <select id="filter-by-document-type" name="document-type">
                <option value=""><?php esc_html_e( 'All types', 'e-connector-for-woocommerce' ); ?></option>
                <?php foreach ( $document_types as $type => $label ) : ?>
                    <option value="<?php echo esc_attr( $type ); ?>" <?php selected( $selected, $type );?>><?php echo esc_html( $label ); ?></option>
                <?php endforeach; ?>
            </select>
        <?php
    }

    /**
     * Handler for admin-post.php requests to download or view the document.
     */
    public function handle_document_action(){
        
        $action = sanitize_title( $_REQUEST['action'] );

        if( ! isset( $action ) || ! in_array( $action, array( 'download_document', 'view_document' ) ) || ! isset( $_REQUEST['_wp_nonce'] ) || ! wp_verify_nonce( $_REQUEST['_wp_nonce'], $action ) ){
            wp_die( __( 'You do not have the rights to do that.', 'e-connector-for-woocommerce' ) );
        }
        if( ! isset( $_REQUEST['document_id'] ) ){
            wp_die( __( 'No document id was provided.', 'e-connector-for-woocommerce' ) );
        }
        
        $user_id         = (int) get_current_user_id();
        $document_id     = (int) $_REQUEST['document_id'];
        $is_owner        = e_connector_for_woocommerce_is_owner( $user_id, $document_id );
        
        if( ! $is_owner || ! user_can( $user_id, 'read_post', $document_id ) ){
            wp_die( __( 'You do not have the right to access this document.', 'e-connector-for-woocommerce' ) );
        }

        $this->send_response( $document_id, $action );
    }

    /**
     * Sends the appropriate document response for a given action.
     * 
     * @param  int     $document_id  Document to send
     * @param  string  $action       Accepts 'download' or 'view'
     */
    public function send_response( $document_id, $action = 'download_document' ){
        $filepath = get_attached_file( $document_id );
        $filename = sanitize_file_name( basename( $filepath ) );
        $mime     = get_post_mime_type( $document_id );
        $filesize = filesize( $filepath );

        ob_clean();
        flush();
        wc_nocache_headers();
        header( 'X-Robots-Tag: noindex, nofollow', true );
		header( 'Content-Type: ' . $mime );
		header( 'Content-Description: File Transfer' );
		header( 'Content-Disposition: inline; filename="' . $filename . '";' );
        if( 'view_document' === $action ){
            header( 'Content-Disposition: inline; filename="' . $filename . '";' );
        } else {
            header( 'Content-Disposition: attachment; filename="' . $filename . '";' );
        }
		header( 'Content-Transfer-Encoding: binary' );
        header( 'Content-Length: ' . $filesize );
        readfile("$filepath");
        exit;
    }
}