<?php

namespace E_connector_For_Woocommerce;
/**
 * Class for handling reading from local database.
 */
class E_Connector_Db {

    /**
     * Name of the SQLITE3 file
     */
    protected $filename;
    
    /**
	 * E-connector uploads
	 */
	protected $uploads_subdir;
    
    /**
     * Actual SQLITE3 object
     */
    protected $db;

    /**
     * Error message
     */
    protected $error = false;

    /**
     * Array of table names
     */
    protected $tables;
    
    /**
     * Constructor
     */
    public function __construct(){
        $this->uploads_subdir = 'econnector_uploads';
        $this->filename       = 'econnector.db';
        $this->db             = $this->maybe_connect();
        $this->tables         = array( 'prices' => 'grilles_prix_manager_v2', );
        $this->table_columns  = array(
            'prices'  => array(
                'GPM_id',
                'source',
                'PriceListId',
                'product_erp_id',
                'product_erp_price',
                'price',
                'reduction_type',
                'prix_definitive',
                'customer_erp_id',
                'customer_cms_id',
                'group_name',
                'id_currency',
                'country_iso2',
                'from_quantity',
                'fromSTR',
                'toSTR',
                'prix_de_base',
                'id_product_attribute',
            ),
        );
    }

    /**
     * Connects to the database
     */
    public function maybe_connect(){
        $upload_dir = wp_upload_dir();
        $filename   = trailingslashit( $upload_dir['basedir'] ) . trailingslashit( $this->uploads_subdir ) . $this->filename;
        $db         = null;
        if( \file_exists( $filename ) ) {
            try {
                $db = new \SQLite3( $filename );
            } catch ( Exception $exception ) {
                $this->error = $exception->getMessage();
            }
        } else {
            $this->error = __( 'Unable to connect to SQLITE3 database. File does not exist.', 'e-connector-for-woocommerce' );
        }
        if( ! empty( $this->error ) ) {
            \error_log( $this->error );
        }
        return $db;
    }

    /**
     * Checks whether the DB is active.
     * 
     * @return  bool
     */
    public function is_active(){
        return is_a( $this->db, 'SQLite3' );
    }

    /**
	 * Uploads a SQLITE DB file from a url
	 * 
	 * @param   string         $url      URL to the file to upload
	 * @return  bool|WP_Error  $success
	 */
	public function update_db( $url ){

		if( ! function_exists( 'download_url' ) ){
			require_once ABSPATH . '/wp-admin/includes/file.php';
		}

        $upload_dir = wp_upload_dir();
        $filename   = trailingslashit( $upload_dir['basedir'] ) . trailingslashit( $this->uploads_subdir ) . $this->filename;
		$tmp_file   = \download_url( $url );
		$success    = false;
		
		if( is_wp_error( $tmp_file ) ){
			$success = $tmp_file;
            error_log( $tmp_file->get_error_message() );
		} else {
			$success = copy( $tmp_file, $filename );
			@unlink( $tmp_file );
		}

		return $success;
	}


     /**
     * Returns the array of tables
     * 
     * @return  array  Array of tables. Format key => table_name
     */
    public function get_tables(){
        return apply_filters( 'e_connector_for_woocommerce_tables', $this->tables );
    }

    /**
     * Returns the array of columns to whitelist against
     * 
     * @param  string  $table    Table to get columns from 
     * @return array   $columns  Column for the given table.
     */
    public function get_table_columns( $table = 'prices' ){
        $columns = array_key_exists( $table, $this->table_columns ) ? $this->table_columns[$table] : false;
        return apply_filters( "e_connector_${table}_table_columns", $columns );
    }

    /**
     * Returns a single table name
     * 
     * @param   string  $table  Slug of the table
     * @return  string          Name of the table
     */
    protected function get_table_name( $table ){
        $tables     = $this->get_tables();
        $table_name = array_key_exists( $table, $tables ) ? $tables[$table] : false;
        return $table_name;
    }

    /**
     * Checks a column actually exists
     * 
     * @return  bool  True if the column exists in the table.
     */
    public function column_exists( $column, $table = 'prices' ){
        $columns = $this->get_table_columns( $table );
        return in_array( $column, $columns );
    }

    /**
     * Fetches rows from a given table
     * 
     * @param   string  $table    The table to fetch entries from.
     * @param   int     $number   Number of entries to fetch. Defaults to 10. Use 0 for all.
     * @param   int     $offset   The number of entries to skip
     * @param   string  $orderby  The column to order results by.
     * @param   string  $order    Ascending or descending.
     * @param   string  $search   A search term
     * @return  array             Array of entries
     */
    public function fetch( $table = 'prices', $number = 10, $offset = 0, $orderby = '', $order = '', $search = '' ){
        $table_name    = $this->get_table_name( $table );
        $table_columns = join( ',', array_map( 'sanitize_title', $this->get_table_columns( $table ) ) );
        if( $table_name && $this->is_active() ){
            $query = sprintf( 'SELECT %s FROM %s', $table_columns, sanitize_title( $table_name ) );

            if( '' != $search ){
                switch ( $table )  {
                    case 'prices':
                        $query .= sprintf( ' WHERE "%s" IN(product_erp_id, customer_erp_id) OR group_name LIKE "%s"', esc_sql( $search ), '%' . esc_sql( $search ) . '%' );
                    break;
                }
            }

            if( ! empty( $orderby ) && $this->validate_column( $orderby, $table ) ) {
                $order  = 'desc' == $order ? 'DESC' : 'ASC';
                $query .= sprintf( ' ORDER BY %s %s', esc_sql( $orderby ), $order );
            }

            if( (int) $number ){
                $query .= ' LIMIT ' . (int) $number;
            }

            if( (int) $offset ){
                $query .= ' OFFSET ' . (int) $offset;
            }

            // Actual query
            $rows    = array();
            $results = $this->db->query( $query );
            while ( $row = $results->fetchArray(SQLITE3_ASSOC) ) {
                $rows[] = $row;
            }
            return $rows;
        }
        return false;
    }

    /**
     * Gets the total number of rows in a table
     * 
     * @param  string  $table   Table name
     * @param  string  $search  Search query
     */
    public function fetch_row_count( $table = 'prices', $search = '' ){
        if( ! $this->is_active() ) return 0;
        $table_name = $this->get_table_name( $table );
        $query      = sprintf( 'SELECT COUNT("GPM_ID") FROM %s', sanitize_title( $table_name ) );
        
        if( '' != $search ){
            switch ( $table )  {
                case 'prices':
                    $query .= sprintf( ' WHERE "%s" IN(product_erp_id, customer_erp_id) OR group_name LIKE "%s"', esc_sql( $search ), '%' . esc_sql( $search ) . '%' );
                break;
            }
        }
        
        $row = $this->is_active() ? $this->db->querySingle( $query ) : 0;        
        return $row;
    }

    /**
     *  Fetches unique column values
     * 
     * @param  string  $table_name
     * @param  string  $column
     * @return  array  $rows
     */
    public function fetch_column_values( $table = 'prices', $column = 'GPM_id' ){
        if( ! $this->is_active() ) return [];
        $table_name = $this->get_table_name( $table );
        $query      = sprintf( 'SELECT DISTINCT %s FROM %s', sanitize_title( $column ), sanitize_title( $table_name ) );
        $rows       = array();
        $results    = $this->db->query( $query );
        while ( $row = $results->fetchArray( SQLITE3_ASSOC ) ) {
            $rows[] = $row;
        }
        return $rows;
    }

    /**
     * Fetches products table row count.
     * 
     * @param  string  $search  Search query
     */
    public function fetch_products_count( $search = '' ){
        return $this->fetch_row_count( 'prices', $search );
    }

    /**
     * Fetches group name values.
     * 
     * @param  string  $search  Search query
     */
    public function fetch_group_name_values(){
        return $this->fetch_column_values( 'prices', 'group_name' );
    }

    /**
     * Validates a column actually exists
     * 
     * @return  bool  True if the column exists in the table.
     */
    public function validate_column( $column, $table = 'prices' ){
        $columns = $this->get_table_columns( $table );
        return in_array( $column, $columns );
    }

    /**
     * Fetches a specific product prices
     * Looks for products with empty customer_erp_id column or matching $customer_id column
     * AND with empty group_name column or matching $customer_group column
     * 
     * @param   string  $product_sku
     * @param   string  $customer_id
     * @param   string  $customer_group
     * @return  array  $rows
     */
    public function fetch_product_prices( $product_sku, $customer_id = '', $customer_group = '' ){
        if( ! $this->is_active() ) return [];

        $table_name = $this->get_table_name( 'prices' );
        $query = sprintf( 'SELECT * FROM %s WHERE product_erp_id="%s"', sanitize_title( $table_name ), esc_sql( $product_sku ) );
        $query .= sprintf( ' AND ( customer_erp_id IS NULL OR customer_cms_id="" OR customer_cms_id="%s" )', esc_sql( $customer_id ) );     
        $query .= sprintf( ' AND ( group_name IS NULL OR group_name="" OR group_name="%s" )', esc_sql( $customer_group ) );
        $query .= sprintf( ' ORDER BY customer_cms_id DESC, group_name DESC, from_quantity DESC' );

        // Actual query
        $rows    = array();
        $results = $this->db->query( $query );
        while ( $row = $results->fetchArray(SQLITE3_ASSOC) ) {
            $rows[] = $row;
        }
        return $rows;
    }
}