<?php
namespace E_connector_For_Woocommerce;
/**
 * Class for handling all content-related actions.
 */
class Product_Manager implements Hookable {
    
    /**
     * Database instance
     */
    protected $db = null;

    /**
     * Products prices
     */
    protected $prices = array();

    /**
     * Applied prices
     */
    protected $applied_prices = array();

    /**
     * Request counter for debugging purposes
     */
    protected $counter = 0;
    
    /**
     * Test whether we're on debugging environment
     */
    protected $debug = false;

    /** 
     * Class constructor 
     */
    public function __construct( $db ){
        $this->db    = $db;
        $this->debug = defined( 'WP_DEBUG' ) && WP_DEBUG;
    }

    /**
     * Registers hooks
     */
    public function register_hooks(){
        if( ! is_admin() && $this->db && $this->db->is_active() ) {
            add_filter( 'woocommerce_product_get_price', array( $this, 'product_price' ), 100, 2 );
            add_filter( 'woocommerce_product_get_regular_price', array( $this, 'product_regular_price' ), 100, 2 );
            add_filter( 'woocommerce_product_is_on_sale', array( $this, 'is_on_sale' ), 100, 2 );
            
            add_filter( 'woocommerce_product_variation_get_price', array( $this, 'product_price' ), 100, 2 );
            add_filter( 'woocommerce_product_variation_get_regular_price', array( $this, 'product_regular_price' ), 100, 2 );
            add_filter( 'woocommerce_product_variation_is_on_sale', array( $this, 'is_on_sale' ), 100, 2 );
            add_filter( 'woocommerce_get_variation_prices_hash', array( $this, 'variation_prices_hash' ), 10, 3 );
            add_filter( 'woocommerce_variation_prices_price', array( $this, 'product_price' ), 100, 2 );
            // add_filter( 'woocommerce_variation_prices_regular_price', array( $this, 'product_price' ), 100, 2 );
            // add_filter( 'woocommerce_variation_prices_sale_price', array( $this, 'product_price' ), 100, 2 );

            add_filter( 'woocommerce_get_price_html', array( $this, 'product_price_html' ), 100, 2 );
            add_action( 'woocommerce_before_calculate_totals', array( $this, 'update_cart_prices' ), 10, 1 );
            add_action( 'woocommerce_cart_loaded_from_session', array( $this, 'update_cart_prices' ), 10, 1 );

            // add_action( 'wp_footer', array( $this, 'debug' ) );
        }
    }

    /**
     * Debug display of data
     * Prints found prices, and applied_prices in the footer.
     */
    public function debug(){
        if( $this->debug ){
            var_dump($this->prices);
            var_dump($this->applied_prices);
        }
    }

    /**
     * Returns product prices data from session.
     * 
     * @return  array  $prices  Prices from session
     */
    public function get_prices(){
        $prices = isset( WC()->session ) ? WC()->session->get( 'e-connector-prices' ) : [];
        if( empty( $prices ) ) $prices = $this->prices;
        return apply_filters( 'e_connector_for_woocommerce_prices', $prices );
    }

    /**
     * Saves applicable prices to session.
     * 
     * @param   array  $prices  Prices to save
     * @return  array  $prices  Prices saved
     */
    public function set_prices( $prices = array() ){
        $prices = apply_filters( 'e_connector_for_woocommerce_before_set_prices', $prices );
        $this->prices = $prices;
        if( ! $this->debug && isset( WC()->session ) ) WC()->session->set( 'e-connector-prices', $prices );
        return $prices;
    }

    /**
     * Returns applied prices from session.
     * 
     * @return  array  $applied_prices  Applied prices from session
     */
    public function get_applied_prices(){
        $applied_prices = isset( WC()->session ) ? WC()->session->get( 'e-connector-applied-prices' ) : [];
        if( empty( $applied_prices ) ) $applied_prices = $this->applied_prices;
        return apply_filters( 'e_connector_for_woocommerce_applied_prices' , $applied_prices );
    }

    /**
     * Saves applied prices to session.
     * 
     * @param   array  $applied_prices  Prices to save
     * @return  array  $applied_prices  Prices saved
     */
    public function set_applied_prices( $applied_prices = array() ){
        $applied_prices = apply_filters( 'e_connector_for_woocommerce_before_set_applied_prices' , $applied_prices );
        if( ! $this->debug && isset( WC()->session ) ) WC()->session->set( 'e-connector-applied-prices', $applied_prices );
        $this->applied_prices = $applied_prices;
        return $applied_prices;
    }

    /**
     * Gets the applied price for a given product
     * 
     * @param   WC_Product  $product
     * @return  array       $applied_price
     */
    public function get_applied_price( $product ){
        $applied_prices = $this->get_applied_prices();
        $product_sku    = $product->get_sku();
        $applied_price  = array();
        if( array_key_exists( $product_sku, $applied_prices ) ){
            $applied_price = $applied_prices[$product_sku];
        }
        return apply_filters( 'e_connector_for_woocommerce_applied_price' , $applied_price, $product );
    }

    /**
     * Sets the applied price for a given product
     * 
     * @param   WC_Product  $product
     * @param   array       $applied_price  Price applied to the product.
     * @return  array       $applied_price
     */
    public function set_applied_price( $product, $applied_price ){
        $applied_prices = $this->get_applied_prices();
        $product_sku    = $product->get_sku();
        $applied_prices[$product_sku] = $applied_price;
        $this->set_applied_prices( $applied_prices );
        return $applied_price;
    }
    
    /**
     * Filters the WC_Product price
     * 
     * @param   string      $price    Woocommerce product price
     * @param   WC_Product  $product
     * @return  string      $price
     */
    public function product_price( $price, $product ){
        if( defined( 'REST_REQUEST' ) && REST_REQUEST ) return $price;
        $applied_price = $this->get_applied_price( $product );
        if( empty( $applied_price ) ){
            $applied_price = $this->update_applied_price( $price, $product );
        }
        if( ! empty( $applied_price ) ){
            $discounted_price = ! empty( $applied_price['applied_price'] ) ? (float) $applied_price['applied_price'] : $this->apply_price( $price, $applied_price, $product );
            $price            = $discounted_price;
        }
        return $price;
    }

    /**
     * Filters the WC_Product regular price
     * 
     * @param   string      $price    Woocommerce product price
     * @param   WC_Product  $product
     * @return  string      $price
     */
    public function product_regular_price( $price, $product ){
        $applied_price = $this->get_applied_price( $product );
        if( $applied_price ){
            $base_price = ! empty( $applied_price['prix_de_base'] ) && $applied_price['prix_de_base'] > 0 ? $applied_price['prix_de_base'] : $price;
            if( $base_price > $price ) $price = $base_price;
        }
        return $price;
    }

    /**
     * Filters whether a product should appear on sale or not
     * Checks the type of the applied price, if any.
     * 
     * @param   bool        $on_sale  
     * @param   WC_Product  $product
     * @return  bool        $on_sale
     */
    public function is_on_sale( $on_sale, $product ){
        $applied_price = $this->get_applied_price( $product );
        if( $applied_price ){
            $on_sale = ! empty( $applied_price['reduction_type'] ) && in_array( $applied_price['reduction_type'], array( 'reduction_percentage', 'reduction_amout', 'amount' ) );
        }
        return $on_sale;
    }

    /**
     * Filters the displayed WooCommerce price  if there are available bulk discounts
     * 
     * @param   string      $price_html  Product object price
     * @param   WC_Product  $product
     * @return  string      $price_html
     */
    public function product_price_html( $price_html, $product ){
        if( defined( 'REST_REQUEST' ) && REST_REQUEST ) return $price_html;
        $product_prices = $this->get_product_prices( $product );
        $qty_discounts  = $this->filter_prices_by_min_discountable_quantity( $product_prices, $product );

        if( ! empty( $qty_discounts ) ){
            $qty_discounts = e_connector_for_woocommerce_array_sort( $qty_discounts, 'from_quantity' );
            $qty_discounts = $this->filter_prices_by_best_price( $qty_discounts, $product );
            if ( ! empty( $qty_discounts ) ){
                $next_qty_discount = $qty_discounts[0]['from_quantity'];
                /* translators: %d number of products */
                $message     = sprintf( __( 'Discount available starting at %d products purchased.', 'e-connector-for-woocommerce' ), (int) $next_qty_discount );
                $price_html .= sprintf( '<br/><span>%s</span>', esc_html( $message ) );
            }
        }
        
        return $price_html;
    }

    /**
     * Makes sure price range are fresh by resetting price hash.
     * 
     * @param   array       $price_hash  Price data before hashing.
     * @param   WC_Product  $product  
     * @param   bool        $for_display
     * @return  array       $price_hash
     */
    public function variation_prices_hash( $price_hash, $product, $for_display ){
        if( $product->is_type( 'variable' ) && $for_display ){
            $transient_name = 'wc_var_prices_' . $product->get_id();
            delete_transient( $transient_name );
            $price_hash = array( false );
        }
        return $price_hash;
    }

    /**
     * Gets applicable prices for the product.
     * Reads from product prices in session 
     * or fetches prices from SQLITE DB for a given product
     * 
     * @param   WC_Product  $product
     * @return  array       $product_prices  Array of applicable prices
     */
    public function get_product_prices( $product ){
        $product_sku    = $product->get_sku();
        $prices         = $this->get_prices();
        $product_prices = array();
        if( array_key_exists( $product_sku, $prices ) ){
            $product_prices = $prices[$product_sku];
        } else {
            if( ! empty( $product_sku ) ){
                $customer_id    = get_current_user_id() ? get_current_user_id() : '';
                $customer_group = get_user_meta( $customer_id, 'e_connector_customer_group', true );
                $product_prices = $this->db->fetch_product_prices( $product_sku, $customer_id, $customer_group );
                $product_prices = $this->filter_prices_by_date( $product_prices );
                $prices[$product_sku] = $product_prices;
                $this->set_prices( $prices );
                $this->counter++;
                if ( $this->debug ) error_log( 'SQLITE DB READ FOR : ' . $product_sku );
            }
        }
        return $product_prices;
    }

    /**
     * Figures out product price to apply, and save it.
     * 
     * @param   string      $price    Woocommerce product price
     * @param   WC_Product  $product 
     */
    public function update_applied_price( $price, $product ){
        $applied_price  = false;
        $product_prices = $this->get_product_prices( $product );
        if( ! empty( $product_prices ) ){
            $product_prices = $this->filter_prices_by_quantity_in_cart( $product_prices, $product );
            if( ! empty( $product_prices ) ){
                // Sort available prices by best price.
                $product_prices   = $this->sort_prices( $product_prices, $price, $product );
                $applicable_price = $product_prices[0];

                // Calculate price based on 'prix_de_base' column if set.
                $discounted_price = ! empty( $applicable_price['applied_price'] ) ? (float) $applicable_price['applied_price'] : $this->apply_price( $price, $applicable_price, $product );
                $base_price       = ! empty( $applicable_price['prix_de_base'] ) && $applicable_price['prix_de_base'] > 0 ? $applicable_price['prix_de_base'] : $price;
                if( (float) $discounted_price < (float) $base_price ){
                    $applied_price = $applicable_price;
                    $this->set_applied_price( $product, $applicable_price );
                }
            }
        }
        return $applied_price;
    }

     /**
     * Sort prices by best applicable price.
     * 
     * @param   array       $product_prices  Array of product prices
     * @param   string      $price           Woocommerce product price
     * @param   WC_Product  $product
     * @return  array       $sorted_prices
     */
    public function sort_prices( $product_prices, $price, $product ){
        $product_prices = array_map( function( $product_price ) use( $price, $product ) {
            $product_price['applied_price'] = $this->apply_price( $price, $product_price, $product );
            return $product_price;
        }, $product_prices );
        $sorted_product_prices = e_connector_for_woocommerce_array_sort( $product_prices, 'applied_price' );
        return $sorted_product_prices;
    }

    /**
     * Updates the applied prices for products in the cart
     * 
     * @param  WC_Cart  $cart  
     */
    public function update_cart_prices( $cart ){

        // If cart is empty, do nothing
        $cart_contents = $cart->get_cart_contents();
        if( empty( $cart_contents ) ) return;

        $should_save = false;
        foreach( $cart_contents as $item_key => $item_data ) {
            $product       = $item_data['data'];
            $price         = $item_data['data']->get_regular_price();
            $applied_price = $this->update_applied_price( $price, $product );
            $should_apply  = empty( $item_data['econnector_applied_price'] ) || $item_data['econnector_applied_price'] !== $applied_price['GPM_id']; 
            if( $applied_price && $should_apply ){
                $item_data['econnector_applied_price'] = sanitize_text_field( $applied_price['GPM_id'] );  
                $cart_contents[$item_key] = $item_data;
                $should_save = true;
            }
        }
        if( $should_save ) {
            $cart->set_cart_contents( $cart_contents );
            $cart->set_session();
        }
    }

    /**
     * Filters the available price data to get only applicable prices on today's date.
     * 
     * @param   array   $product_prices  Array of product prices
     * @param   string  $customer_id
     * @param   string  $customer_group
     * @return  array       $product_prices  Filtered prices 
     */
    public function filter_prices_by_date( $product_prices ){
        return $this->filter_prices( 'date', $product_prices );
    }

    /**
     * Filters product prices by applicable quantity.
     * Checks the 'from_quantity' column
     * If product is in cart, checks quantity in cart is enough.
     * If not, returns prices where 'from_quantity' is empty or 1.
     * 
     * @param   array       $product_prices  Prices to filter
     * @param   WC_Product  $product         WC Product if needed
     * @return  array       $product_prices  Filtered prices 
     */
    public function filter_prices_by_quantity_in_cart( $product_prices, $product ){        
        return $this->filter_prices( 'quantity_in_cart', $product_prices, array( 'product' => $product ) );
    }

    /**
     * Filters product prices by minimum applicable quantity for a discount.
     * Checks the 'from_quantity' column
     * If product is in cart, checks quantity in below new 'from_quantity'.
     * If not, returns prices where 'from_quantity' is greater than 1.
     * 
     * @param   array       $product_prices  Prices to filter
     * @param   WC_Product  $product         WC Product if needed
     * @return  array       $product_prices  Filtered prices 
     */
    public function filter_prices_by_min_discountable_quantity( $product_prices, $product ){        
        return $this->filter_prices( 'min_discountable_qty', $product_prices, array( 'product' => $product ) );
    }

     /**
     * Filters product prices by applicable type.
     * Checks the 'reduction_type' column
     * 
     * @param   array       $product_prices  Prices to filter
     * @param   string      $type            Type to filter on if needed
     * @return  array       $product_prices  Filtered prices  
     */
    public function filter_prices_by_type( $product_prices, $type = null ){        
        return $this->filter_prices( 'type', $product_prices, array( 'type' => $type ) );
    }

     /**
     * Filters product prices by better applied price.
     * Removes product prices where discount is not better than current price 
     * 
     * @param   array       $product_prices  Prices to filter
     * @param   WC_Product  $product         WC Product if needed
     * @return  array       $product_prices  Filtered prices  
     */
    public function filter_prices_by_best_price( $product_prices, $product = null ){        
        return $this->filter_prices( 'best_price', $product_prices, array( 'product' => $product ) );
    }

    /**
     * Filter applicable prices
     * 
     * @param   string  $field           Criteria to filter on.
     * @param   array   $product_prices  Prices to filter
     * @param   array   $args            Argument to filter by. Accepts 'product', 'type'
     * @return  array   $product_prices  Filtered prices  
     */
    public function filter_prices( $field, $product_prices, $args = array() ){        
        switch ( $field ) {
            case 'type':
                if( ! $args['type'] ) break;
                    $product_prices = array_filter( $product_prices, function( $product_price ) use ( $type ){
                        return ! empty( $product_price['reduction_type'] ) && $args['type'] === $product_price['reduction_type'];  
                    } );
                break;
            case 'quantity_in_cart':
            case 'min_discountable_qty':
                if( ! $args['product'] ) break;
                $cart_product_id = $this->is_product_in_cart( $args['product'] );
                $quantities      = isset( WC()->cart ) ? WC()->cart->get_cart_item_quantities() : [];
                $product_id      = (int) $args['product']->get_ID();        
                $product_prices = array_filter( $product_prices, function( $product_price ) use( $cart_product_id, $quantities, $product_id, $field ) {
                    $min_qty  = ! empty( $product_price['from_quantity'] ) ? (int) $product_price['from_quantity'] : 1;
                    $item_qty = ! empty( $quantities ) && ! empty( $quantities[$product_id] ) ? (int) $quantities[$product_id] : 0;
                    if( 'min_discountable_qty' === $field ){
                        return $item_qty ? $item_qty < $min_qty : $min_qty > 1;
                    }
                    return $item_qty ? $item_qty >= $min_qty : 1 === $min_qty;
                } );
                break;
            case 'date':
                $product_prices = array_filter( $product_prices, function( $product_price ){
                    $valid = true;
                    if( (bool) $product_price['fromSTR'] ){
                        $from = date_create_from_format( 'd/m/Y', $product_price['fromSTR'] );
                        $from_timestamp = $from->getTimestamp();
                        if( time() <= $from_timestamp ) $valid = false;
                    }
                    if( (bool) $product_price['toSTR'] ){
                        $to = date_create_from_format( 'd/m/Y', $product_price['toSTR'] );
                        $to_timestamp = $to->getTimestamp();
                        if( time() >= $to_timestamp ) $valid = false;
                    }
                    return $valid;
                } );
                break;
            case 'best_price':
                if( ! $args['product'] ) break;
                $product        = $args['product'];
                $current_price  = $product->get_price();
                $regular_price  = $product->get_regular_price();
                $product_prices = array_filter( $product_prices, function( $product_price ) use( $product, $current_price, $regular_price ) {
                    $discounted_price = $this->apply_price( $regular_price, $product_price, $product );
                    return $discounted_price < $current_price;
                } );
                break;
        }       
        return array_values( $product_prices );
    }

    /**
     * Applies a product price
     * 
     * @param   string      $price             Original WooCommerce price
     * @param   array       $applicable_price  
     * @param   WC_Product  $product
     * @return  string      $price  
     */
    public function apply_price( $price, $applicable_price, $product ){

        // Calculate price base on 'prix_de_base' column if set.
        $base_price = ! empty( $applicable_price['prix_de_base'] ) && $applicable_price['prix_de_base'] > 0 ? (float) $applicable_price['prix_de_base'] : $price;
        $applied_price = $price;
        if( ! empty( $applicable_price['price'] ) ){
            switch ( $applicable_price['reduction_type'] ) {
                case 'reduction_percentage':
                    if( $applicable_price['price'] <= 100 ){
                        $percent       = ( 100 - (float) $applicable_price['price'] ) / 100;
                        $applied_price = (float) $base_price * $percent;
                    }
                    break;
                case 'reduction_amount':
                case 'amount':
                    $applied_price = (float) $base_price - (float) $applicable_price['price'];
                    break;
                case 'price':
                    $applied_price = $applicable_price['price'];
                    break;
            }
        }

        return apply_filters( 'e_connector_for_woocommerce_applied_price', $applied_price, $price, $base_price, $applicable_price, $product );
    }


    /**
     * Checks whether a product is in cart
     * 
     * @param   WC_Product  $product
     * @return  string      $in_cart  Cart product id
     */
    public function is_product_in_cart( $product ){
        $product_cart_id = isset( WC()->cart ) ? WC()->cart->generate_cart_id( $product->get_id() ) : 0;
        $in_cart = isset( WC()->cart ) ? WC()->cart->find_product_in_cart( $product_cart_id ) : false;
        return apply_filters( 'e_connector_for_woocommerce_is_product_in_cart', $in_cart, $product );
    }
}