<?php
namespace E_connector_For_Woocommerce;
/**
 * Registers and handle new REST API endpoint for uploading documents.
 */
class WC_REST_Documents_Controller extends WC_REST_Uploads_Controller {

	/**
	 * REST base for the ressource
	 */
	protected $rest_base;
	
	/**
	 * User subdirectory name
	 */
	protected $user_subdir;

	/**
	 * Initializes the controller
	 */
	public function __construct(){
		parent::__construct();
		$this->rest_base   = 'documents';
		$this->user_subdir = '';
	}

	/**
	 * Registers the REST routes
	 */
	public function register_routes() {
		register_rest_route( $this->namespace, '/' . $this->rest_base,
			array(
				'methods'   => \WP_REST_Server::CREATABLE,
				'callback'  => array( $this, 'upload_document' ),
				'permission_callback' => array( $this, 'upload_item_permissions_check' ),
				'args'      => array(
					'user_id' => array(
						'required'          => true,
						'validate_callback' => array( $this, 'validate_param' ),
					),
					'total_paid' => array(
						'validate_callback' => array( $this, 'validate_param' ),
					),
					'reference' => array(
						'validate_callback' => array( $this, 'validate_param' ),
					),
					'document_type' => array(
						'validate_callback' => array( $this, 'validate_param' ),
					),
				),
			)
		);
	}

	/**
	 *  Gets the user folder inside the Econnector uploads directory
	 * 
	 * @param   int     $user_id  User ID
	 * @return  string  $subdir   User upload subdirectory
	 */
	public function get_user_subdir( $user_id ){
		$user = get_userdata( $user_id );
		$hash = '';
		if( $user ){
			$username = sanitize_text_field( $user->user_login );
			$hash     = \sha1( $username );
		}
		return trailingslashit( $this->uploads_subdir ) . $hash;
	}

	/**
	 * Validates POST request parameters
	 * 
	 * @param   string           $value    Value to validate
	 * @param   WP_REST_Request  $request  Request object  
	 * @param   string           $key      Request parameter key
	 * @return  bool             $valid    Whether value is valid
	 */
	public function validate_param( $value, $request, $key ){
		switch ( $key ) {
			case 'user_id':
			case 'document_type':
				return is_numeric( $value );
				break;
			case 'total_paid':
			case 'reference':
				return is_string( $value );
				break;
		}
		return false;
	}

	/**
	 * Handler for the POST request to 
	 * @param  WP_REST_Request  $request  Request object
	 */
	public function upload_document( $request ) {
		$user_id       = $request->get_param( 'user_id' );
		$reference     = ! empty( $request->get_param( 'reference' ) ) ? sanitize_text_field( $request->get_param('reference') ) : false;
		$total_paid    = ! empty( $request->get_param( 'total_paid' ) ) ? sanitize_text_field( $request->get_param('total_paid') ) : '';
		$document_type = ! empty( $request->get_param( 'document_type' ) ) ? sanitize_text_field( $request->get_param( 'document_type' ) ) : '' ;
		$form_file     = ! empty( $request->get_file_params() ) && ! empty( $request->get_file_params()['file'] ) ? $request->get_file_params()['file'] : false;
		$encoded_file  = $request->get_param( 'encoded_file' );

		// Temporarily update the upload directory
		$this->user_subdir = $this->get_user_subdir( $user_id );
		add_filter( 'upload_dir', array( $this, 'upload_dir' ), 10, 1 );

		// Upload either file if present
		$uploaded_file_id = 0;
		if( $encoded_file ){
			$uploaded_file_id = $this->upload_encoded_file( $encoded_file, $reference, $user_id );
		}
		if( $form_file ){
			$uploaded_file_id = $this->upload_form_file( $form_file, $user_id );
		}

		if( is_wp_error( $uploaded_file_id ) ){
			return $uploaded_file_id;
		}

		// Updates user and attachment meta data
		if( $uploaded_file_id ){
			// Generate thumbnail
			$this->generate_uploaded_file_metadata( $uploaded_file_id );
			$user_files = get_user_meta( $user_id, '_econnector_documents', true );
			if( ! $user_files ){
				$user_files = array();
			}
			$user_files[] = (int) $uploaded_file_id;
			update_user_meta( $user_id, '_econnector_documents', $user_files );
			
			update_post_meta( $uploaded_file_id, '_econnector_owner', (int) $user_id );
			update_post_meta( $uploaded_file_id, '_total_paid', $total_paid );
			update_post_meta( $uploaded_file_id, '_reference', $reference );
			update_post_meta( $uploaded_file_id, '_document_type', $document_type );

			$sent = $this->send_email( $uploaded_file_id, $user_id );
		}
		remove_filter( 'upload_dir', array( $this, 'upload_dir' ), 10 );
		
		$this->maybe_install_protection( 'index.php' );
		$this->maybe_install_protection( '.htaccess' );
		return $uploaded_file_id;
	}

	/**
	 * Uploads a file submitted via standard $_FILES
	 * 
	 * @param   string        $form_file         File to upload
	 * @return  int|WP_Error  $uploaded_file_id  ID of the attachment created or WP_Error
	 */
	public function upload_form_file( $form_file, $user_id = false ){

		// Check mime type and extension
		$type = wp_check_filetype( $form_file['name'], $this->get_allowed_mime_types() );
		if( ! $type['ext'] || ! $type['type'] ){
			return new \WP_Error( 'econnector_rest_invalid_file', __( 'Sorry, the file you try to upload is invalid.', 'e-connector-for-woocommerce' ), $type );
		}

		if( ! function_exists( 'media_handle_upload' ) ){
			require_once ABSPATH . '/wp-admin/includes/file.php';
			require_once ABSPATH . '/wp-admin/includes/media.php';
			require_once ABSPATH . '/wp-admin/includes/image.php';
		}
		
		$uploaded_file_id = media_handle_upload( 'file', 0, array( 'post_author' => (int) $user_id ) );
		return $uploaded_file_id;
	}
	
	/**
	 * Uploads a base64-encoded submitted file
	 * 
	 * @param   string        $encoded_file      Contents of the file
	 * @return  int|WP_Error  $uploaded_file_id  ID of the attachment created or WP_Error
	 */
	public function upload_encoded_file( $encoded_file, $name = 'document', $user_id = false ){
		$contents = base64_decode( $encoded_file );
		$mime     = finfo_buffer( finfo_open( FILEINFO_MIME ), $contents, FILEINFO_MIME_TYPE );
		$ext      = $this->get_extension_or_mime( $mime, 'ext', true );
		$ext      = 'htm' === $ext ? 'html' : $ext;

		if( 'application/octet-stream' === $mime ){
			$substr = substr( $contents, 0, 32 );
			if( false !== stripos( $substr, '%pdf' ) ){
				$mime = 'application/pdf';
				$ext  = 'pdf';
			}
		}

		if( ! in_array( $mime, array_values( $this->get_allowed_mime_types() ) ) || ! $ext ){
			return new \WP_Error( 'econnector_rest_invalid_file', __( 'Sorry, the file you try to upload is invalid.', 'e-connector-for-woocommerce' ), $mime );
		}
		
		$name          = ! empty( $name ) ? sanitize_file_name( $name ) : 'document';
		$filename      = sprintf( '%s.%s', $name, sanitize_key( $ext ) );
		$uploaded_file = wp_upload_bits( $filename, null, $contents );
		
		if( ! empty( $uploaded_file['error'] ) ){
			return new \WP_Error( 'econnector_rest_upload_error', __( 'Sorry, there was an error uploading the file.', 'e-connector-for-woocommerce' ), $uploaded_file['error'] );
		}
		// Create attachment file
		$data = array(
			'post_author'    => (int) $user_id,
			'post_mime_type' => $uploaded_file['type'],
			'guid'           => $uploaded_file['url'],
			'post_parent'    => 0,
			'post_title'     => sanitize_text_field( $filename ),
			'post_content'   => '',
			'post_excerpt'   => '',
		);
		$uploaded_file_id = wp_insert_attachment( $data, $uploaded_file['file'], 0, true );
		return $uploaded_file_id;
	}

	/**
	 * Generates uploaded file metadata and thumbnails
	 * 
	 * @param   int  $uploaded_file_id  File to generate meta for
	 * @return  int  $meta_id           Updated meta id.
	 */
	function generate_uploaded_file_metadata( $uploaded_file_id ){
		if( ! function_exists( 'wp_generate_attachment_metadata' ) ){
			require_once ABSPATH . 'wp-admin/includes/image.php';
		}			
		$uploaded_filepath = \get_attached_file( $uploaded_file_id );
		if( ! $uploaded_filepath ){
			return false;
		}
		$metadata = \wp_generate_attachment_metadata( $uploaded_file_id, $uploaded_filepath );
		if ( is_wp_error( $metadata ) || empty( $metadata ) ) {
			return false;
		}
		return wp_update_attachment_metadata( $uploaded_file_id, $metadata );
	}

	/**
	 * Flatten a $_FILES array for ease of use
	 * 
	 * @param   array  $files   Array of files
	 * @return  array  $result  Reordered array  
	 */
	function format_files_array( $files ) {
		$result = array();
		foreach( $files as $key1 => $value1 ){
			foreach( $value1 as $key2 => $value2 ){
				$result[$key2][$key1] = $value2;
			}
		}
		return $result;
	}
	
	/**
	 * Send email notifications.
	 * 
	 * @param  int  $document_id  ID of the document just uploaded
	 * @param  int  $user_id      ID of the owner
	 */
	public function send_email( $document_id, $user_id = false ){
		$emails = WC()->mailer()->get_emails();
		if( empty( $emails ) ){
			return false;
		}
		if( array_key_exists( 'WC_Email_New_Document', $emails ) ){
			$emails['WC_Email_New_Document']->trigger( $document_id, $user_id );
		}
		if( array_key_exists( 'WC_Email_Customer_New_Document', $emails ) ){
			$emails['WC_Email_Customer_New_Document']->trigger( $document_id, $user_id );
		}
		return true;
	}
}