<?php
namespace E_connector_For_Woocommerce;
/**
 * Registers common functions for upload handling via WooCommerce Rest API.
 */
class WC_REST_Uploads_Controller extends \WC_REST_Controller {
    
    /**
	 * Namespace of the controller routes
	 */
	protected $namespace;

	/**
	 * REST base for the ressource
	 */
	protected $rest_base;
	
	/**
	 * Allowed file types
	 */
	protected $allowed_mime_types;
	
	/**
	 * Subdirectory name
	 */
	protected $uploads_subdir;

	/**
	 * User subdirectory name
	 */
	protected $user_subdir;

	/**
	 * Initializes the controller
	 */
	public function __construct(){
		add_filter( 'upload_mimes', array( $this, 'allowed_upload_mimes' ), 10, 2 );
		$this->namespace          = 'wc/econnector/v1';
		$this->rest_base          = 'documents';
		$this->uploads_subdir     = 'econnector_uploads';
		$this->allowed_mime_types = get_allowed_mime_types();
		$this->user_subdir        = '';
	}

	/**
	 * Returns the allowed file types and their extensions.
	 */
	public function get_allowed_mime_types(){
		return apply_filters( 'econnector_allowed_mime_types', $this->allowed_mime_types );
	}

	/**
	 * Adds necessary authorization for uploading.
	 * 
	 * @param   array    $mimes  Allowed ext => mime
	 * @param   WP_User  $user  WP User
	 * @return  array    $mimes  
	 */
	public function allowed_upload_mimes( $mimes, $user = false ){
		if( ! user_can( $user, 'unfiltered_html' ) || ! in_array( 'text/html', array_values( $mimes ) ) ){
			$mimes['htm|html'] = 'text/html';
		}
		return $mimes;
	}

	/**
	 * Filters the upload directory to get to E-Connector's folder
	 * 
	 * @param   array  $upload_dir  Array of upload directory data
	 * @return  array  $upload_dir
	 */
	public function upload_dir( $upload_dir ){
		$upload_dir['subdir'] = ! empty( $this->user_subdir ) ? $this->user_subdir : $this->uploads_subdir ;
		$upload_dir['path']   = trailingslashit( $upload_dir['basedir'] ) . $upload_dir['subdir'];
		$upload_dir['url']    = trailingslashit( $upload_dir['baseurl'] ) . $upload_dir['subdir'];
		return $upload_dir;
	}

	/**
	 * Checks for upload permissions
	 */
	public function upload_item_permissions_check(){
		if( ! current_user_can( 'upload_files' ) ){
			return new \WP_Error( 'econnector_rest_cannot_upload', __( 'Sorry, you are not allowed to upload documents.', 'e-connector-for-woocommerce' ), array( 'status' => rest_authorization_required_code() ) );
		}
		return true;
	}

	/**
	 * Reads allowed mime types and extension, and get a suitable extension for passed in mime type
	 * 
	 * @param   string  $ext_or_mime  Mime type to get extension for, or extension to get Mime for.
	 * @param   string  $field        Whether to get a 'mime' or an 'ext'.
	 * @param   bool    $single       Whether to return only a single extension when multiple are possible.
	 * @return  string  $result       Extension or mime
	 */
	public function get_extension_or_mime( $ext_or_mime, $field = 'mime', $single = false ) {
		$allowed_mimes = $this->get_allowed_mime_types();
		if( 'ext' === $field ){
			$allowed_mimes = array_flip( $allowed_mimes );
		}

		$result = false;
		foreach ( $allowed_mimes as $key => $value ) {
			if( $key === $ext_or_mime || ( false !== strpos( $key, $ext_or_mime ) ) ){
				$result = $value;
			}
		}
		if( $result && 'ext' === $field && $single ){
			$result = explode( '|', $result )[0];
		}
		return $result;
	}

	/**
	 * Checks existence and installs index.php, and .htaccess files, and create them if needed.
	 * 
	 * @param   string     $file     Filename. Accepts 'index.php' or '.htaccess'
	 * @return  int|false  $success  File size or false
	 */
	public function maybe_install_protection( $file = 'index.php' ){
		$rootfilename   = trailingslashit( $this->uploads_subdir ) . $file;
		$userfilename   = trailingslashit( $this->user_subdir ) . $file;
		$upload_dir     = \wp_upload_dir();
		$basedir        = $upload_dir['basedir'];
		$rootfullname   = trailingslashit( $basedir ) . $rootfilename;
		$userfullname   = trailingslashit( $basedir ) . $userfilename;
		$filecontent    = $this->get_protection_file_content( $file );
		$success        = false;
		
		// Maybe install root file
		if( ! \file_exists( $rootfullname ) && $filecontent ) {
			$success = file_put_contents( $rootfullname, $filecontent );
		}
		
		// Maybe install user file
		if( ! empty( $this->user_subdir ) && ! \file_exists( $userfullname ) && $filecontent ) {
			$success = file_put_contents( $userfullname, $filecontent );
		}
		
		return $success;
	}

	/**
	 * Returns content of protection files.
	 * 
	 * @param   string  $file     Filename. Accepts 'index.php' or '.htaccess'
	 * @return  string  $content  String content. Default false
	 */
	public function get_protection_file_content( $file = 'index.php' ){
		$content = false;
		switch ( $file ) {
			case 'index.php':
				$content =  "<?php // Silence is golden \n ";
				break;
			case '.htaccess':
				$content   = "# BEGIN ECONNECTOR \n";
				$content  .= "Options -Indexes\n";
				$content  .= "<FilesMatch \"\.(html|pdf)$\">\n";
				$content  .= "Order Deny,Allow\n";
				$content  .= "Deny from all\n";
				$content  .= "</FilesMatch>\n";
				$content  .= "# END ECONNECTOR \n";
				break;
		}
		return $content;
	}
}