<?php
/**
 * Contains heper functions available throughout the plugin
 */

/**
 * Returns whether the site uses SQLITE3 price table or not.
 * 
 * @return  bool 
 */
function e_connector_for_woocommerce_should_use_db(){
    $settings = get_option( 'e_connector_settings' );
    return ! empty( $settings ) && ! empty( $settings['should_use_db'] ) && 'yes' === $settings['should_use_db'];
}

/**
 * Insert a value or key/value pair after a specific key in an array.  
 * If key doesn't exist, value is appended to the end of the array.
 *
 * @param   array   $array  Original array to add to
 * @param   string  $key    Key to add new item after
 * @param   array   $new    Array to add after.
 * @return  array
 */
function e_connector_for_woocommerce_array_insert( $array, $key, $new, $after = true  ) {
	$keys  = array_keys( $array );
	$index = array_search( $key, $keys );
    $pos   = false === $index ? count( $array ) : $index;
    $pos   = $after ? $pos + 1 : $pos;
	return array_merge( array_slice( $array, 0, $pos ), $new, array_slice( $array, $pos ) );
}

/**
 * Sort a multi-dimensional array by column name.  
 *
 * @param   array   $array   Original array to add to
 * @param   string  $column  Column name to sort by
 * @return  array
 */
function e_connector_for_woocommerce_array_sort( $array, $column, $order = SORT_ASC ) {
    $column = array_column( $array, $column );
    if( ! empty( $column ) ){
        array_multisort( $column, $order, $array );
    }
	return $array;
}

/**
 * Returns the path to a template file.
 * Looks first if the file exists in the `econnector/` folder in the child theme,
 * then in the parent's theme `econnector/` folder,
 * finally in the default plugin's template directory
 *
 * @param   string  $slug     The template we're looking for
 * @return  string  $located  The path to the template file if found.
 */
function e_connector_for_woocommerce_get_template( $slug ) {
    $located = '';
    
    $basename = sanitize_file_name( basename( $slug ) . '.php' );
    $template_name = trailingslashit( dirname( $slug ) ) . $basename;

    if ( file_exists( STYLESHEETPATH . '/woocommerce/' . $template_name ) ) {
        $located = STYLESHEETPATH . '/woocommerce/' . $template_name;
    } elseif ( file_exists( TEMPLATEPATH . '/woocommerce/' . $template_name ) ) {
        $located = TEMPLATEPATH . '/woocommerce/' . $template_name;
    } elseif ( file_exists( E_CONNECTOR_FOR_WOOCOMMERCE_PATH . 'templates/' . $template_name ) ) {
        $located = E_CONNECTOR_FOR_WOOCOMMERCE_PATH . 'templates/' . $template_name;
    }

    return str_replace( '..', '', $located ) ;
}

/**
 * Returns available document types
 * 
 * @return  array  $types  Document types. Array of code => label
 */
function e_connector_for_woocommerce_get_document_types(){
    $types = array(
        '20' => __( 'Quote', 'e-connector-for-woocommerce' ),
        '21' => __( 'Order', 'e-connector-for-woocommerce' ),
        '22' => __( 'Shipping label', 'e-connector-for-woocommerce' ),
        '23' => __( 'Invoice', 'e-connector-for-woocommerce' )
    );
    return apply_filters( 'e_connector_for_woocommerce_document_types', $types );
}

/**
 * Returns whether a document is owned by a user
 * 
 * @param   int   $document_id  Id of the document to check the owner of. Default to current post
 * @param   int   $user         User id to check against. Default to current user
 * @return  bool  $is_owner     True if $user_id owns $document_id
 */
function e_connector_for_woocommerce_is_owner( $user_id = false, $document_id = false ){
    if( ! $document_id ){
        $document_id = get_the_ID();
    }
    if( ! $user_id ){
        $user_id = get_current_user_id();
    }
    $document_owner  = (int) get_post_meta( $document_id, '_econnector_owner', true );
    $owner_documents = get_user_meta( $user_id, '_econnector_documents', true );
    $is_owner        = ! empty( $owner_documents ) && in_array( $document_id, $owner_documents ) && $document_owner === $user_id;
    return apply_filters( 'e_connector_for_woocommerce_is_owner', $is_owner, $document_id, $user_id );
}

/**
 * Returns the label for a given document type
 * 
 * @param  string  $type  Type code.
 * @return  string  $label  Type label
 */
function e_connector_for_woocommerce_get_document_type_label( $type = false ){
    $types = e_connector_for_woocommerce_get_document_types();
    $label = '';
    if( $type && ! empty( $types ) && array_key_exists( $type, $types ) ){
        $label = $types[$type];
    }
    return apply_filters( 'e_connector_for_woocommerce_document_type_label', $label, $type );
}

/**
 * Returns available icons
 * 
 * @return  array  $icons  Icons. Array of slug => svg
 */
function e_connector_for_woocommerce_get_icons(){
    // Uses Zondicons icons
    $icons = array(
        'download' => '<svg class="icon icon-download" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 20 20"><path d="M13 8V2H7v6H2l8 8 8-8h-5zM0 18h20v2H0v-2z"/></svg>',
        'view'     => '<svg class="icon icon-view" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 20 20"><path d="M.2 10a11 11 0 0 1 19.6 0A11 11 0 0 1 .2 10zm9.8 4a4 4 0 1 0 0-8 4 4 0 0 0 0 8zm0-2a2 2 0 1 1 0-4 2 2 0 0 1 0 4z"/></svg>',
    );
    return apply_filters( 'e_connector_for_woocommerce_icons', $icons );
}

/**
 * Returns the svg icon for a given icon slug
 * 
 * @param   string  $slug  Icon slug.
 * @return  string  $icon  SVG icon
 */
function e_connector_for_woocommerce_get_icon( $slug = false ){
    $icons = e_connector_for_woocommerce_get_icons();
    $icon  = '';
    if( $slug && ! empty( $icons ) && array_key_exists( $slug, $icons ) ){
        $icon = $icons[$slug];
    }
    return apply_filters( 'e_connector_for_woocommerce_icon', $icon, $slug );
}